"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Aggregate = void 0;
class Aggregate {
    mongoModel;
    pipe;
    constructor(mongoModel = {}, pipe = []) {
        this.mongoModel = mongoModel;
        this.pipe = pipe;
    }
    /**
     * Returns the current pipeline
     */
    pipeline = () => this.pipe;
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // STAGES
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    project = (proj) => new Aggregate(this.mongoModel, [...this.pipe, { $project: proj }]);
    match = (match) => new Aggregate(this.mongoModel, [...this.pipe, { $match: match }]);
    unset = (unset) => new Aggregate(this.mongoModel, [...this.pipe, { $unset: unset }]);
    sort = (sort) => new Aggregate(this.mongoModel, [...this.pipe, { $sort: sort }]);
    limit = (limit) => new Aggregate(this.mongoModel, [...this.pipe, { $limit: limit }]);
    unwind = (unwind) => new Aggregate(this.mongoModel, [...this.pipe, { $unwind: unwind }]);
    count = (key) => new Aggregate(this.mongoModel, [...this.pipe, { $count: key }]);
    lookup = (lookupQuery, subAggregate) => {
        if ('localField' in lookupQuery)
            return new Aggregate(this.mongoModel, [
                ...this.pipe,
                { $lookup: { as: lookupQuery.from, ...lookupQuery } },
            ]);
        const lookup = {
            as: lookupQuery.from,
            ...lookupQuery,
        };
        let paramLookupAggregate = {};
        Object.keys(lookupQuery.let || {}).forEach((key) => {
            paramLookupAggregate[key] = `$$${key}`;
        });
        lookup.pipeline = subAggregate(new Aggregate(), paramLookupAggregate).pipe;
        return new Aggregate(this.mongoModel, [...this.pipe, { $lookup: lookup }]);
    };
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // EXEC
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * Executes the aggregate pipeline on the currently bound Model
     */
    exec = async () => {
        if (this.pipe.length === 0)
            throw new Error(`You can't use exec on an empty aggregate pipeline. Add some stages or use the paginate function to act like a find mongoose method.`);
        return await this.mongoModel.aggregate(this.pipe);
    };
    /**
     * Executes the aggregate pipeline on the currently bound Model by paginating the results
     *
     * @param1 skip
     * @param2 limit
     */
    paginate = async (skip, limit) => {
        this.pipe.push({
            $facet: {
                metadata: [{ $count: 'total' }],
                data: [{ $skip: skip }, { $limit: limit }],
            },
        });
        const res = await this.mongoModel.aggregate(this.pipe);
        return {
            data: res[0].data,
            total: res[0].metadata[0]?.total || 0,
            skip,
            limit,
        };
    };
    countExec = async (key) => {
        this.pipe.push({ $count: key });
        const res = await this.mongoModel.aggregate(this.pipe);
        return { [key]: 0, ...res[0] };
    };
}
exports.Aggregate = Aggregate;
